<nav class="navbar navbar-expand-lg navbar-light bg-light">
    <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarToggler" aria-controls="navbarToggler" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
    </button>
    <a class="navbar-brand" href="{{ url_for('zubbi.index') }}">Zubbi</a>

    <div class="collapse navbar-collapse" id="navbarToggler">
        <ul class="navbar-nav mr-auto mt-2 mt-lg-0">
            {{ macros.nav_item('zubbi.index', 'Home', 'fas fa-home') }}
            {{ macros.nav_item('zubbi.how-to', 'How-to', 'fas fa-question-circle') }}
        </ul>
    </div>
</nav>
