<div class="highlight-rst notranslate"><div class="highlight"><pre><span></span><span class="p">..</span> <span class="ow">code-block</span><span class="p">::</span> python  # or any other language like bash|yaml|...

   import this
</pre></div>
</div>
