<footer class="footer" align="center">
    <span class="value">Version: {{ meta.version }}</span>
    <span class="value">
        <a href="https://github.com/bmwcarit/zubbi">
            <i class="fab fa-github pr-1"></i>Zubbi
        </a>
    </span>
</footer>
