<!-- Bootstrap CSS -->
<link rel="stylesheet" href="{{ url_for('static', filename='css/bootstrap.min.css') }}" >
<link rel="stylesheet" href="{{ url_for('static', filename='css/fontawesome-all.min.css') }}" >
<link rel="stylesheet" href="{{ url_for('static', filename='css/jquery-ui.min.css') }}" >
<link rel="stylesheet" href="{{ url_for('static', filename='pygments.css') }}">
<link rel="stylesheet" href="{{ url_for('static', filename='style.css') }}">
